
/*
 ******************************************
 **    © by Widgetschmie.de 2005-2008    **
 ******************************************
 */


var showHash = new Object();
var lastParagraphIndex = 0;
var lastActiveShow = null;
var magnifiedImage = null;				// only one magnified Image ("screenshot" of a movie) can be shown at a time


function idFromCoredataID(core) {
	return core ? core.substr(13).replace(/[^\d\w]+/g, "_") : null;
}

function showsFromIDs(showIDs)
{
	showHash = new Object();
	var shows = new Array();
	if(showIDs)
	{
		for (var i = 0; i < showIDs.length; i++)
		{
			var someshow = new Show(showIDs[i]);
			showHash[someshow.coredataID] = someshow;
			if(someshow.loadData()) {
				shows.push(someshow);
			}
		}
	}
	return shows;
}


// *****************************
// ** Class for a single show **
// *****************************

function Show(coredataID) {
	if(! this instanceof Show) {
		return new Show();
	}
	
	// Attributes
	this.coredataID = coredataID;
	this.id = idFromCoredataID(coredataID);
	this.title = title ? title : locStr("Name der Sendung fehlt");
	this.startstamp;
	this.endstamp;
	this.duration;				// in minutes
	this.fractionDone = 0;		// -1 = Alert set, 0 = not started (but alert not set), 1 = over, float between 0 and 1 = fraction already played
	this.isRunning = false;
	this.channel;				// id of the channel
	this.tunerChannel;			// assigned tuner channel for this.channel
	this.actors;
	this.originalTitle;
	this.infoURL;
	this.genre;
	this.programImage;
	this.programImageData;
	this.moreInfo;
	this.year;
	this.country;
	this.rating;
	
	this.p = null;							// holds the mainRow - DIV
	this.piechartCanvas = null;				// the piechart canvas
	this.tunerCanvas = null;				// tuner icon canvas
	this.ibuttonCanvas = null;				// canvas for the moreinfo-button
	this.ratingStarCanvas = null;			// canvas for the rating stars
	this.moreInfoShown = false;				// BOOL
	this.showDescriptionShown = false;		// BOOL
	this.recordingScheduled = false;		// BOOL
	this.applicableFilters;					// the IDs of the filters that are true for this show (only for shows that need visual highlighting of a substring)
	this.filterMatchesInMoreinfo = false;	// BOOL
	
	// *******
	// Methods
	
	// load data from Plugin
	this.loadData = function() {
		if(window.widget) {
			var details = FernsehPlugin.showDetailsForID(this.coredataID);
			if(!details)
				return null;
			
			this.title = details[0];
			this.startstamp = Math.abs(details[1]);
			this.endstamp = Math.abs(details[2]);
			this.channel = details[3];
			this.duration = Math.abs(details[4]);			// in minutes
			this.moreInfo = details[5].replace(/_/g, "—");
			this.infoURL = details[6];
			this.fractionDone = parseFloat(details[7]);
			this.genre = details[8];
			this.programID = details[9];
			this.applicableFilters = (details[10] && details[10].length > 0) ? details[10].split(" ") : false;
			
			// calculations (highly sophisticated)
			this.isRunning = (this.fractionDone > 0 && this.fractionDone < 1);
			
			// alert
			this.alertSet = FernsehPlugin.alarmExistsForShow(this.coredataID);
			this.fractionDone = (!this.isRunning && this.alertSet) ? -1 : this.fractionDone;
			
			this.recordingScheduled = (tunerUsed && availableChannels[this.channel]) ? FernsehPlugin.recordingTimerExists(availableChannels[this.channel].assignedTuner, ""+this.startstamp) : false;
			
			if ("1" == details[11]) // do we have extended details?
			{
				this.originalTitle = FernsehPlugin.originaltitleForShowWithID(this.coredataID);
				this.year = FernsehPlugin.yearForShowWithID(this.coredataID);
				this.country = FernsehPlugin.countryForShowWithID(this.coredataID);
				this.rating = FernsehPlugin.ratingForShowWithID(this.coredataID);
			}
			
			return true;
		}
	}
	
	this.getMainRow = function() {
		var p = document.createElement('div');
		p.setAttribute('id', 'main_' + this.id);
		
		
		// ****
		// Left Info Panel
		var lInfoPanel = document.createElement('div');
		lInfoPanel.setAttribute('class', "leftinfopanel");
		
		// piechart
		this.createPiechartCanvas();
		lInfoPanel.appendChild(this.piechartCanvas);
		
		// tuner button
		if(tunerUsed) {
			this.createTunerCanvas();
			lInfoPanel.appendChild(this.tunerCanvas);
		}
		
		// Starttime
		var time = document.createElement('span');
		time.setAttribute('class', "time");
		time.setAttribute('onclick', "updateTVListing('" + this.startstamp + "')");
		time.innerHTML = formatTime(this.startstamp);
		lInfoPanel.appendChild(time);
		
		p.appendChild(lInfoPanel);
		
		
		// ****
		// Right Info Panel
		var rInfoPanel = document.createElement('div');
		rInfoPanel.setAttribute('class', "rightinfopanel");
		
		// Channel
		var channel = document.createElement('span');
		channel.setAttribute('class', "channel");
		channel.setAttribute('onclick', "changeToChannelOnlyView('" + this.channel + "', '" + this.startstamp + "')");
		channel.innerHTML = availableChannels[this.channel] ? availableChannels[this.channel].getName() : this.channel;
		rInfoPanel.appendChild(channel);
		
		// Toggle more info - button
		var ibutcanvas = document.createElement('canvas');
		ibutcanvas.setAttribute('class', "toggleshortinfo");
		ibutcanvas.setAttribute('width', canvasWidthHeight);
		ibutcanvas.setAttribute('height', canvasWidthHeight);
		ibutcanvas.setAttribute('lang', this.coredataID);
		ibutcanvas.setAttribute('onclick', "toggleMoreInfo(event)");
		
		this.ibuttonCanvas = ibutcanvas;
		rInfoPanel.appendChild(this.ibuttonCanvas);
		
		p.appendChild(rInfoPanel);
		
		
		// ****
		// Title
		var title = this.getTitle();
		if(debugMode) {
			title = formatDate(this.startstamp);
		}
		var title_pod = document.createElement("div");
		title_pod.setAttribute("class", "titlePod");
		title_pod.innerHTML = title;
		title_pod.setAttribute('onclick', "toggleShowDescription('" + this.coredataID + "')");
		
		p.appendChild(title_pod);
		
		this.p = p;
		return p;
	}
	
	this.getMagazineRow = function(starting_date) {
		if(!_now) {
			_now = new Date();		_now.setSeconds(0);		_now.setMilliseconds(0);		// should be set by the calling method
		}
		
		var show = document.createElement("div");
		
		var from = new Date();				from.setTime(this.startstamp);
		var till = new Date();				till.setTime(this.endstamp);
		
		var start_min = (from.getTime() - starting_date.getTime()) / 1000 / 60;
		var hidden_min = 0;
		var show_height = this.duration;
		
		// show started yesterday
		if(from < starting_date) {
			hidden_min = -1 * start_min;
			show_height -= hidden_min;
			start_min = 0;
			show.setAttribute('style', "border-top-style:dotted; -webkit-border-top-left-radius:0; -webkit-border-top-right-radius:0;");
		}
		
		// show runs longer than we show
		if(till.getTime() > (starting_date.getTime() + 86400000)) {
			hidden_min = (till.getTime() - (starting_date.getTime() + 86400000)) / 1000 / 60;
			show_height -= hidden_min;
			hidden_min = 0;
			show.setAttribute('style', "border-bottom-style:dotted; -webkit-border-bottom-left-radius:0; -webkit-border-bottom-right-radius:0;");
		}
		
		// running show - cover the area already done
		if((from < _now) && (till > _now)) {
			var minutes_done = Math.round((_now.getTime() - this.startstamp) / 1000 / 60) - hidden_min;
			if(minutes_done > 0) {
				var cover = document.createElement("div");
				cover.setAttribute("class", "magazineDoneCover");
				cover.setAttribute("style", "height:" + (minutes_done * magazineViewScaleFactor) + "px;");
				show.appendChild(cover);
			}
		}
		
		// format show DOM element
		var paddingComp = 12;				// -12 = padding + border, 
		if(this.duration < 10) {			// shows running shorter than 10 minutes
			show.style.padding = '2px';
			show.style.webkitBorderRadius = "0.25em";
			paddingComp = 4;
		}
		show.setAttribute("class", ((_now >= till) ? "magazineShowDone" : "magazineShow"));
		show.style.top = (start_min * magazineViewScaleFactor + 2) + "px";
		show.style.height = ((show_height * magazineViewScaleFactor) - paddingComp - 4) + "px";		// -4 = 2px top, 2px bottom-"margin"
		show.setAttribute('onclick', "toggleShowDescription('" + this.coredataID + "')");
		
		// Show title (and canvas-buttons)
		var box = document.createElement('div');
		box.setAttribute('class', "magazineShowBox");
		
		var title = document.createElement("div");
		title.setAttribute("class", "magazineShowTitle");
		if(this.fractionDone <= 0) {
			this.createPiechartCanvas();
			title.appendChild(this.piechartCanvas);
		}
		if(tunerUsed && (this.fractionDone < 1)) {
			this.createTunerCanvas();
			if(this.tunerChannel) {
				title.appendChild(this.tunerCanvas);
			}
		}
		
		var titlespan = document.createElement('span');
		titlespan.innerHTML = this.getTitle();
		title.appendChild(titlespan);		// formatDate(this.startstamp)
		box.appendChild(title);
		
		// show time and infos
		var time = document.createElement('div');
		time.setAttribute('class', "magazineShowTime");
		time.innerHTML = formatTime(from) + " " + locStr('bis') + " " + formatTime(till);
		box.appendChild(time);
		
		var info = document.createElement('div');
		info.setAttribute('class', "magazineShowInfo");
		info.innerHTML = "(" + this.duration + " " + locStr('Minuten') + ")<br />" + this.genre;
		box.appendChild(info);
		
		show.appendChild(box);
		this.p = show;
		
		// shown in drawer?
		if(lastActiveShow == this.coredataID) {
			this.showInDrawer();
		}
		
		return show;
	}
	
	
	this.getMoreinfoRow = function() {
		var cont = document.createElement('div');
		cont.setAttribute('id', 'desc_' + this.id);
		cont.setAttribute('class', "descRow");
		
		var desc = document.createElement('div');
		desc.setAttribute('id', 'desccontent_' + this.id);
		desc.setAttribute('class', "descContent");
		
		// showlength
		var showlength = document.createElement('div');
		showlength.setAttribute("class", "showlengthPanel");
		showlength.setAttribute("style", "padding-left:" + (1* canvasWidthHeight + 2) + "px;");
		showlength.innerHTML = this.duration + ' ' + locStr("Minuten") + ' ' + locStr("bis") + ' ' + formatTime(this.endstamp);
		
		// moreinfo text
		var desc_data = document.createElement("div");
		desc_data.setAttribute("class", "descData");
		desc_data.setAttribute("style", "padding-left:" + (1* canvasWidthHeight + 2) + "px;");
		var txt = this.genre + (this.originalTitle ? " (\"" + this.originalTitle + "\")" : '');
		var highlightStrings = new Array();
		
		// highlight: search
		if(isSearchResult && inArray('title', searchOptions)) {
			highlightStrings = ourSearchString.split(/\s+/);
		}
		
		// highlight: filters
		if(this.applicableFilters) {
			for(var i = 0; i < this.applicableFilters.length; i++) {
				var fltr = filterWithID(this.applicableFilters[i]);
				if(fltr && fltr.mark) {
					highlightStrings.push(fltr.string);
				}
			}
		}
		
		this.filterMatchesInMoreinfo = false;
		if(highlightStrings.length > 0) {
			var newtxt = FernsehPlugin.highlightStrings(highlightStrings.join("\n"), txt);
			this.filterMatchesInMoreinfo = (newtxt != txt);
			txt = newtxt;
		}
		desc_data.innerHTML = txt;
		
		this.moreInfoShown = showShortDesc || this.filterMatchesInMoreinfo;
		if(!this.moreInfoShown) {
			cont.style.height = '0px';
		}
		
		var clear = document.createElement('div');
		clear.setAttribute('style', "clear:both;");
		
		// compose
		desc.appendChild(showlength);
		desc.appendChild(desc_data);
		//desc.appendChild(clear);
		cont.appendChild(desc);
		
		return cont;
	}
	
	this.display = function() {
		var p = this.getMainRow();
		var m = this.getMoreinfoRow();
		p.setAttribute('class', ((0 == lastParagraphIndex % 2) ? 'mainRow' : 'mainRow altColor'));
		m.setAttribute('class', (0 == lastParagraphIndex % 2) ? 'descRow' : 'descRow altColor');
		
		// attach
		var parent = getObj('content');
		parent.appendChild(p);
		parent.appendChild(m);
		lastParagraphIndex++;
		
		// shown in drawer?
		if(lastActiveShow == this.coredataID) {
			this.showInDrawer();
		}
	}
	
	// returns the title - formatted
	this.getTitle = function() {
		var highlightStrings = new Array();
		
		// search
		if(isSearchResult && inArray('title', searchOptions)) {
			highlightStrings = ourSearchString.split(/\s+/);
		}
		
		// filters
		if(this.applicableFilters) {
			for(var i = 0; i < this.applicableFilters.length; i++) {
				var fltr = filterWithID(this.applicableFilters[i]);
				if(fltr && fltr.mark) {
					highlightStrings.push(fltr.string);
				}
			}
		}
		
		var ttl = this.title;
		if(highlightStrings.length > 0) {
			ttl = FernsehPlugin.highlightStrings(highlightStrings.join("\n"), ttl);
			
			// nothing was marked - show a badge
			if(ttl == this.title) {
				ttl = "<span class=\"searchResult\">&nbsp;☆&nbsp;</span>&nbsp;" + ttl;
			}
		}
		
		return ttl;
	}
	
	
	// draws the canvasses for this show. It knows what to do, simply call it without argument or pass one of the special-arguments if you know better
	this.drawCanvas = function(special) {
		if(('switchingChannel' == special) && this.tunerCanvas) {
			drawClosingEye(this.tunerCanvas.getContext("2d"), this.showDescriptionShown);
			return;
		}
		else if(('settingRecording' == special) && this.tunerCanvas) {
			var flt = 0.5;
			
			drawCassette(this.tunerCanvas.getContext("2d"), flt, this.showDescriptionShown);
			return;
		}
		else if(('togglingMoreinfo' == special) && this.ibuttonCanvas) {
			drawiButton(this.ibuttonCanvas.getContext("2d"), this.moreInfoShown);
			return;
		}
		else if(('showInDrawer' == special) && this.ratingStarCanvas) {
			var context = this.ratingStarCanvas.getContext("2d");
			
			for(var i = 0; i < 5; i++) {
				drawStar(context, i, (i < this.rating));
			}
			return;
		}
		
		if(this.piechartCanvas) {
			this.piechartCanvas.width = canvasWidthHeight;
			this.piechartCanvas.height = canvasWidthHeight;
			var context = this.piechartCanvas.getContext("2d");
			
			switch(this.fractionDone) {
				case -1:		drawAlarmIcon(context, true, this.showDescriptionShown);					break;
				case 0:			drawAlarmIcon(context, false, this.showDescriptionShown);					break;
				case 1:			drawShowOver(context, this.showDescriptionShown);							break;
				default:		drawPiechart(context, this.fractionDone, this.showDescriptionShown);		break;
			}
		}
		if(this.tunerCanvas) {
			this.tunerCanvas.width = canvasWidthHeight;
			this.tunerCanvas.height = canvasWidthHeight;
			var context = this.tunerCanvas.getContext("2d");
			
			if(this.tunerChannel) {
				if(this.isRunning) {
					drawEye(context, (this.channel == tunerIsCurrentlyPlaying), this.showDescriptionShown);
				}
				else if((this.fractionDone <= 0) && tunerSupportsRecording(tunerUsed)) {
					drawCassette(context, this.recordingScheduled, this.showDescriptionShown);
				}
			}
		}
		if(this.ibuttonCanvas) {
			this.ibuttonCanvas.width = canvasWidthHeight;
			this.ibuttonCanvas.height = canvasWidthHeight;
			var context = this.ibuttonCanvas.getContext("2d");
			
			drawiButton(context, this.moreInfoShown);
		}
	}
	
	
	// creates the piechart canvas and assigns functions and stuff
	this.createPiechartCanvas = function() {
		var piechart = document.createElement('canvas');
		piechart.setAttribute('id', 'piechart_' + this.id);
		piechart.setAttribute('width', canvasWidthHeight);
		piechart.setAttribute('height', canvasWidthHeight);
		
		if(this.fractionDone <= 0) {						// ----- alert icon
			piechart.setAttribute('onclick', "toggleAlertForShow('" + this.coredataID + "')");
		}
		else if(this.isRunning) {							// ----- show is running; pie-chart
			var title = minutesAsString(this.duration) + ' ' + locStr('Gesamtlaufzeit') + "\n" + 
						minutesAsString(Math.floor(this.duration * this.fractionDone)) + ' ' + locStr('gelaufen') + "\n" + 
						minutesAsString(Math.ceil(this.duration * (1 - this.fractionDone))) + ' ' + locStr('verbleibend');
			
			piechart.setAttribute('title', title);
		}
		
		this.piechartCanvas = piechart;
	}
	
	// creates the canvas to switch the channel
	this.createTunerCanvas = function() {
		var tunerCanvas = document.createElement('canvas');
		tunerCanvas.setAttribute('id', "tunerCanvas_" + this.channel);
		tunerCanvas.setAttribute('width', canvasWidthHeight);
		tunerCanvas.setAttribute('height', canvasWidthHeight);
		tunerCanvas.setAttribute('lang', this.coredataID);
		
		this.tunerChannel = availableChannels[this.channel] ? availableChannels[this.channel].assignedTuner : null;
		
		// show is currently running
		if(this.tunerChannel && this.isRunning) {
			tunerCanvas.setAttribute('onclick', "switchTVToChannel(\"" + this.coredataID + "\")");
			tunerCanvas.setAttribute('title', locStr('Sender einschalten'));
		}
		
		// show can be recorded; only EyeTV and FormacDVB support the tape icon!
		else if((this.fractionDone <= 0) && tunerSupportsRecording(tunerUsed) && this.tunerChannel) {
			tunerCanvas.setAttribute("onclick", "toggleRecordingTimer('" + this.coredataID + "')");
			tunerCanvas.setAttribute('title', this.recordingScheduled ? locStr('Sendung wird aufgenommen') : locStr('Sendung aufnehmen'));
		}
		
		this.tunerCanvas = tunerCanvas;
	}
	
	
	// returns the link to TVStar
	this.getMoreInfoLink = function() {
		var tvstarimg = document.createElement('img');
		tvstarimg.src = "Images/TVstar_logo.png";
		tvstarimg.setAttribute("class", "moreInfoLogo");
		tvstarimg.style.height = canvasWidthHeight + "px";
		tvstarimg.setAttribute("onclick", "openURL('" + this.infoURL + "')");
		
		var link = document.createElement('div');
		link.setAttribute("class", "moreInfoLink");
		
		link.appendChild(document.createTextNode(locStr("Mehr Infos auf")));
		link.appendChild(tvstarimg);
		
		return link;
	}
	
	
	// returns a canvas with the Stars for the rating of the show
	this.getRatingStars = function() {
		if(!this.rating) {
			return null;
		}
		if(this.ratingStarCanvas) {
			return this.ratingStarCanvas;
		}
		
		var canvas = document.createElement('canvas');
		canvas.setAttribute('class', "ratingStarsCanvas");
		canvas.setAttribute('width', 5 * canvasWidthHeight);
		canvas.setAttribute('height', canvasWidthHeight);
		if(this.infoURL) {
			canvas.setAttribute('onclick', "javascript:openURL('" + this.infoURL + "')");
		}
		
		this.ratingStarCanvas = canvas;
		return this.ratingStarCanvas;
	}
	
	
	// toggles the alert for this show
	this.toggleAlert = function() {
		var remove = FernsehPlugin.alarmExistsForShow(this.coredataID);
		if(remove) {
			FernsehPlugin.removeAlarmForShow(this.coredataID);
		}
		else {
			var eyetvchannel = availableChannels[this.channel].assignedTuner;
			if (undefined == eyetvchannel) eyetvchannel = "";
			FernsehPlugin.setAlarmForShow(this.coredataID);
		}
		
		// button
		this.alertSet = remove ? false : true;
		this.fractionDone = remove ? 0 : -1;
		this.drawCanvas();
		
		if(window.event) {
			window.event.stopPropagation();
			window.event.preventDefault();
		}
	}
	
	
	// toggles the recording
	this.toggleRecordingTimer = function() {
		if(tunerUsed && tunerSupportsRecording(tunerUsed) && this.tunerCanvas) {
			//this.drawCanvas('settingRecording');
			
			if(this.recordingScheduled) {
				if(deleteRecordTimerForShow(availableChannels[this.channel].getTuner(), this.title, this.duration, this.startstamp)) {
					this.tunerCanvas.setAttribute('title', locStr('Sendung aufnehmen'));
					this.recordingScheduled = false;
				}
				else {
					showDialog(locStr("Beim Entfernen der programmierten Aufnahme ist ein Fehler aufgetreten"), locStr("OK"), blendDialogOut);
				}
			}
			else {
				if(setRecordTimerForShow(availableChannels[this.channel].getTuner(), this.title, this.duration, this.startstamp)) {
					this.tunerCanvas.setAttribute('title', locStr('Sendung wird aufgenommen'));
					this.recordingScheduled = true;
				}
			}
			
			this.drawCanvas();
		}
	}
	
	
	// toggles more info
	this.toggleMoreInfo = function(duration) {
		var desc = getObj("desc_" + this.id);
		
		if(desc) {
			var curr_height = desc.offsetHeight;
			var content_height = getObj("desccontent_" + this.id).offsetHeight;
			
			this.descAnim = { obj:desc, start:(new Date()).getTime(), animTime:duration, from:curr_height, to:0 }
			this.descAnim.to = this.moreInfoShown ? 0 : content_height;
			
			this.moreInfoShown = (this.descAnim.to > 0);
			this.drawCanvas('togglingMoreinfo');
			this.animateMoreInfo((0 == duration) ? true : false);
		}
	}
	
	this.animateMoreInfo = function(no_thumb_recalc) {
		if(this.descAnim && this.descAnim.obj) {
			var now = (new Date()).getTime();
			var end = this.descAnim.start + 1* this.descAnim.animTime;
			
			// animate
			if(now < end) {
				var delta = (now - this.descAnim.start) / this.descAnim.animTime;
				var factor = ((delta * delta) * 3.0) - ((delta * delta * delta) * 2.0);
				
				var curr = ((this.descAnim.to - this.descAnim.from) * factor) + (1* this.descAnim.from);
				this.descAnim.obj.style.height = curr + 'px';
				
				setTimeout("showHash['" + this.coredataID + "'].animateMoreInfo(" + no_thumb_recalc + ")", 10);
			}
			
			// end animation
			else {
				this.descAnim.obj.style.height = this.descAnim.to + 'px';
				this.descAnim = null;
				if(!no_thumb_recalc) {
					calculateAndShowThumb('myScrollBar', getObj('content'));
				}
			}
		}
	}
	
	
	// toggles the show description
	this.toggleShowDescription = function() {
		if(lastActiveShow == this.coredataID) {		// hide info for this show when clicking the title of this show again
			this.clearFromDrawer(true);
		}
		else {
			this.showInDrawer();					// show info for this show
		}
	}
	
	// show this shows info in the drawer
	this.showInDrawer = function() {
		if(lastActiveShow && (lastActiveShow != this.coredataID) && showHash[lastActiveShow]) {
			showHash[lastActiveShow].clearFromDrawer(false);
		}
		
		// mark current show
		lastActiveShow = this.coredataID;
		this.p.setAttribute('class', this.p.getAttribute('class') + (('magazine' == displaySetting['mode']) ? ' activeShow' : ' activeRow'));
		this.showDescriptionShown = true;
		this.drawCanvas();
		
		// compose XHTML
		var description = document.createElement('div');
		var infoURLDisplayed = false;
		
		// load program image
		this.programImageData = FernsehPlugin.base64ImageDataForShowWithID(this.coredataID);
		if(this.programImageData && ("#no_image" != this.programImageData)) {
			this.programImage = document.createElement('img');
			this.programImage.setAttribute('class', "drawerImage");
			this.programImage.setAttribute('onclick', "magnifyImage(\"" + this.coredataID + "\")");
			this.programImage.src = ("#loading" == this.programImageData) ? "Images/detailImagePlaceholder.png" : "data:image/png;base64," + this.programImageData;		// !! add new image
			description.appendChild(this.programImage);
		}
		
		if(this.moreInfo) {
			var container = document.createElement("div");
			var textbox = document.createElement('div');
			
			// matching searchstrings or filters?
			var highlightStrings = new Array();
			var highlightCastStrings = new Array();
			
			if(isSearchResult) {
				highlightStrings.push(ourSearchString.split(/\s+/).join("\n"));
			}
			if(isSearchResult && (inArray('director', searchOptions) || inArray('actors', searchOptions))) {
				highlightCastStrings.push(ourSearchString.split(/\s+/).join("\n"));
			}
			
			if(this.applicableFilters) {
				for(var i = 0; i < this.applicableFilters.length; i++) {
					var fltr = filterWithID(this.applicableFilters[i]);
					if(fltr && fltr.mark) {
						highlightStrings.push(fltr.string);
						if(fltr.also_cast) {
							highlightCastStrings.push(fltr.string);
						}
					}
				}
			}
			
			// main text
			var moreInfoText = this.moreInfo;
			if(highlightStrings.length > 0 && ((isSearchResult && inArray('description', searchOptions)) || !isSearchResult)) {
				moreInfoText = FernsehPlugin.highlightStrings(highlightStrings.join("\n"), moreInfoText);
			}
			
			if(this.originalTitle) {
				var origTtl = this.originalTitle;
				if(highlightStrings.length > 0 && inArray('title', searchOptions)) {
					origTtl = FernsehPlugin.highlightStrings(highlightStrings.join("\n"), this.originalTitle);
				}
				moreInfoText = "(\"" + origTtl + "\"" + (this.country ? ", " + this.country : '') + (this.year ? " " + this.year : '') + ")<br /><br />" + moreInfoText;
			}
			
			textbox.innerHTML = moreInfoText;
			container.appendChild(textbox);
			
			// add extended details - actors and directors
			if(FernsehPlugin.hasExtendedDetailsForShowWithID(this.coredataID)) {
				
				// directors
				var directors = FernsehPlugin.directorsForShowWithID(this.coredataID);
				if(directors && (directors != "#none") && (directors.length > 0)) {
					var dircontainer = document.createElement('div');
					dircontainer.setAttribute('class', "directorContainer");
					dircontainer.innerHTML = "<i>" + locStr("Regie") + "</i><br />";
					
					for(var i = 0; i < directors.length; i++) {
						var directorName = FernsehPlugin.nameOfPersonWithID(directors[i]);
						if(highlightCastStrings.length > 0) {
							directorName = FernsehPlugin.highlightStrings(highlightCastStrings.join("\n"), directorName);
						}
						dircontainer.innerHTML += "<a href=\"javascript:showPersonShows('" + directors[i] + "')\">" + directorName + "</a><br />";
					}
					
					// add the TVStar Rating and the moreInfo Link
					if(this.rating) {
						dircontainer.appendChild(this.getRatingStars());
					}
					
					if(this.infoURL) {
						dircontainer.appendChild(this.getMoreInfoLink());
						infoURLDisplayed = true;
					}
					
					// movie database (like imdb)
					if(this.originalTitle) {
						var imdblogo = document.createElement("img");
						imdblogo.setAttribute("class", "imdb_link");
						imdblogo.setAttribute("src", ('ofdb' == movieDatabase) ? "Images/OFDb_logo.png" : "Images/IMDb_logo.png");
						imdblogo.setAttribute("onclick", "gotoMovieDatabase(\"" + this.coredataID + "\")");
						dircontainer.appendChild(imdblogo);
					}
					
					container.appendChild(dircontainer);
				}
				
				// actors
				var actors = FernsehPlugin.actorsAndRolesForShowWithID(this.coredataID);
				if(actors && (actors != "#none") && (actors.length > 0)) {
					var actorcontainer = document.createElement('div');
					actorcontainer.setAttribute('class', "actorContainer");
					actorcontainer.innerHTML = "<i>" + locStr("Schauspieler") + "</i><br />";
					
					for(var i = 0; i < actors.length; i++) {
						var actorShowDetails = FernsehPlugin.actorAndRoleForPersonShowID(actors[i]);
						
						var actor = actorShowDetails[0];
						var role = actorShowDetails[1];
						
						if(highlightCastStrings.length > 0) {
							actor = FernsehPlugin.highlightStrings(highlightCastStrings.join("\n"), actor);
							role = FernsehPlugin.highlightStrings(highlightCastStrings.join("\n"), role);
						}
						actorcontainer.innerHTML += "<a href=\"javascript:showPersonShows('" + actorShowDetails[2] + "')\">" + actor + "</a>: " + role + "<br />";
					}
					
					container.appendChild(actorcontainer);
				}
			}
			
			
			description.appendChild(container);
		}
		
		if(this.infoURL && !infoURLDisplayed) {
			if(this.rating) {
				dircontainer.appendChild(this.getRatingStars());
			}
			
			description.appendChild(this.getMoreInfoLink());
		}
		showInDrawer(description);
		this.drawCanvas('showInDrawer');
		setTimeout("resizeDrawer()", 20);
	}
	
	// hides the drawer und unmarks the current show
	this.clearFromDrawer = function(close_drawer) {
		if(this.p) {
			this.p.setAttribute('class', this.p.getAttribute('class').replace(/ *active(Row|Show)/, ''));		// 'activeShow' and 'activeRow'
		}
		lastActiveShow = null;
		this.showDescriptionShown = false;
		this.ratingStarCanvas = null;
		this.drawCanvas();
		
		if(close_drawer) {
			showInDrawer();
			resizeDrawer('close');
		}
	}
	
	
	// get the imdb (or other) link
	this.gotoMovieDB = function() {
		var link = "http://www.imdb.com/Find?select=Titles&for=" + this.originalTitle + "&Go.x=15&Go.y=11&Go=Go";		// default = imdb
		switch(movieDatabase) {
			case 'german.imdb':		link = "http://german.imdb.com/Find?select=Titles&for=" + this.originalTitle + "&Go.x=15&Go.y=11&Go=Go";
									break;
			case 'ofdb':			var loc = (""+(document.location)).split("/");
									loc[loc.length-1] = "OFDBSearch.html?Kat=OTitel&SText=";
									loc = loc.join("/");
									link = loc + encodeURIComponent(this.originalTitle);
									break;
		}
		
		openURL(link);
	}
	
	// magnify the image
	this.magnifyImage = function() {
		if(this.programImageData && ("#loading" != this.programImageData) && ("#no_image" != this.programImageData)) {
			magnifiedImage = document.createElement('div');
			magnifiedImage.setAttribute('class', "toolbarListParent");
			magnifiedImage.setAttribute('onclick', "closeMagnifiedImage(\"" + this.coredataID + "\")");
			var img = document.createElement('img');
			img.setAttribute('class', "magnifiedImage");
			img.src = "data:image/png;base64," + this.programImageData;
			
			magnifiedImage.appendChild(img);
			getObj("frontDiv").appendChild(magnifiedImage);
		}
	}
	
	this.closeMagnifiedImage = function() {
		if(magnifiedImage) {
			magnifiedImage.parentNode.removeChild(magnifiedImage);
			magnifiedImage = null;
		}
	}
	
	
	// finish initialization
	return this;
}


// toggles the alert of the given show
function toggleAlertForShow(coredataID) {
	if(showHash[coredataID]) {
		showHash[coredataID].toggleAlert();
	}
}


// toggles more info for the given show
function toggleMoreInfo(event) {
	var coredataID = event.target ? event.target.getAttribute('lang') : '0';
	var duration = event.shiftKey ? 3000 : 250;
	
	// show or hide all
	if(event.altKey) {
		var top_offset = event.target ? (event.target.parentNode.parentNode.offsetTop - (-1 * parseInt(getObj('content').style.top))) : 0;		// compensate scrolled offset
		
		var do_hide = showHash[coredataID].moreInfoShown;
		for(var cid in showHash) {
			if((do_hide && showHash[cid].moreInfoShown) || (!do_hide && !showHash[cid].moreInfoShown)) {
				showHash[cid].toggleMoreInfo(event.shiftKey ? duration : 0);
			}
		}
		
		// scroll to the previously saved position - the clicked "i" must remain under the mouse cursor
		if(top_offset > 0) {
			var surplus = top_offset - event.target.parentNode.parentNode.offsetTop;
			surplus = (surplus > 0) ? 0 : surplus;
			getObj('content').style.top = surplus + "px";
		}
		calculateAndShowThumb('myScrollBar', getObj('content'));
	}
	
	// show or hide the clicked one
	else {
		if(showHash[coredataID]) {
			showHash[coredataID].toggleMoreInfo(duration);
		}
	}
}

// called from the plugin once loading an image in the background is done
// updates the "drawer" content if the show's info is currently being shown
function updateImageForShow(showID, imageData) {
	var show = showHash[showID];
	if(show) {
		if("#no_image" == imageData) {
			if(show.programImage && show.programImage.parentNode) {
				show.programImage.parentNode.removeChild(show.programImage);
			}
		}
		else {
			show.programImageData = imageData;
			
			// do drawer magic here.
			if(show.programImage) {
				show.programImage.src = "data:image/png;base64," + show.programImageData;
			}
			else {
				alert("Show " + showID + " has no programImage although we loaded data for it!");
			}
		}
		
		// let the drawer animation finish before resizing
		var timeout = drawer.stop ? (drawer.stop - (new Date()).getTime()) : 0;
		timeout = Math.max(timeout, 0) + 20;
		setTimeout("resizeDrawer()", timeout);
	}
}


function gotoMovieDatabase(showID) {
	if(showHash[showID]) {
		showHash[showID].gotoMovieDB();
	}
	else {
		alert("gotoMovieDatabase(): Show with id " + showID + " not found");
	}
}


function magnifyImage(showID) {
	if(showHash[showID]) {
		showHash[showID].magnifyImage();
	}
	else {
		alert("magnifyImage(): Show with id " + showID + " not found");
	}
}

function closeMagnifiedImage(showID) {
	if(showHash[showID]) {
		showHash[showID].closeMagnifiedImage();
	}
	else {
		alert("closeMagnifiedImage(): Show with id " + showID + " not found");
	}
}


// toggles the show description
function toggleShowDescription(coredataID) {
	if(showHash[coredataID]) {
		showHash[coredataID].toggleShowDescription();
	}
}

